@@ -1,7 +1,7 @@
 #-*- coding: ISO-8859-1 -*-
 # pysqlite2/test/__init__.py: the package containing the test suite
 #
-# Copyright (C) 2004-2006 Gerhard Hring <gh@ghaering.de>
+# Copyright (C) 2004-2007 Gerhard Hring <gh@ghaering.de>
 #
 # This file is part of pysqlite.
 #
@@ -21,14 +21,29 @@
 #    misrepresented as being the original software.
 # 3. This notice may not be removed or altered from any source distribution.
 
+import os, sys
 import unittest
+
+if os.path.exists("extended_setup.py"):
+    print "-" * 75
+    print "You should not run the test suite from the pysqlite build directory."
+    print "This does not work well because the extension module cannot be found."
+    print "Just run the test suite from somewhere else, please!"
+    print "-" * 75
+    sys.exit(1)
+
 from pysqlite2.test import dbapi, types, userfunctions, factory, transactions,\
     hooks, regression
+from pysqlite2 import dbapi2 as sqlite
 
 def suite():
-    return unittest.TestSuite(
-        (dbapi.suite(), types.suite(), userfunctions.suite(), factory.suite(),\
-         transactions.suite(), hooks.suite(), regression.suite()))
+    tests = [dbapi.suite(), types.suite(), userfunctions.suite(),
+      factory.suite(), transactions.suite(), hooks.suite(), regression.suite()]
+    if sys.version_info >= (2, 5, 0):
+        from pysqlite2.test import py25tests
+        tests.append(py25tests.suite())
+
+    return unittest.TestSuite(tuple(tests))
 
 def test():
     runner = unittest.TextTestRunner()
